%---------------------------------------------------------------------
% TEST_SBR.m: restoration of 17 Gaussian features from noisy data
%
% Use of the Single Best Replacement (SBR) algorithm: call to Matlab
% function sbr_ld.m
%
%	The program displays two figures showing:
%	(1) the locations of the true spikes and the spikes estimated by SBR. 
%	(2) the data signal together with its approximation
%
%---------------------------------------------------------------------
% This is a supplementary material to the paper:
% 
% From Bernoulli-Gaussian deconvolution to sparse signal restoration, 
% by Charles Soussen, Jerome Idier, David Brie, and Junbo Duan. 
% IEEE Trans. Signal Process., vol. 59, no. 10, pp. 4572-4584, Oct. 2011.
% 
% Copyright (c) 2011 IEEE
% Contact information: Charles.Soussen@univ-lorraine.fr
%---------------------------------------------------------------------
clear all;
close all;
clc;
warning off;
randn('state',4);  
rand('state',4);  

M = 3000;       % size of observation
sigma = 50;     % standard deviation of Gaussian features

% 17 spikes are expected
mu = .02; % mu=1 => 6 spikes are recovered
          % mu=0.2 => 11 spikes
          % mu=0.05 => 14 spikes 
          % mu=0.02 => 16 spikes WITH REMOVALS
          % mu=0.01 => 20 spikes WITH REMOVALS
          % mu=0.02 is an acceptable the empirical trade-off

%-------------------------------------------------
%----- Generation of impulse response ------------
%-------------------------------------------------
% Generation of a centered truncated Gaussian of support [1-M:M-1]
val_x = (1-M):(M-1);
% indices of samples at which the Gaussian is non zero
support = find(abs(val_x)<=3*sigma);  
% IMPULSE RESPONSE  
h = exp(-val_x(support).^2/(2*sigma*sigma));  % followed by zeros...
% normalization: energy = 1
factor = norm(h,2);  
h = h/factor;

ind_start = ceil(1+3*sigma);  % centre of the first possible Gaussian
ind_end = M-ind_start+1;      % centre of the last possible Gaussian

disp('BUILDING OBSERVATION MATRIX H... (takes a few seconds)'); 
%------------------- Construction of matrix H, sparse -----
P = length(h);
N = ind_end-ind_start+1;  % size of x, or N=M-P+1, with P=length(h)
col = zeros(M,1);
col(1:P) = h';
lin = zeros(N,1);
lin(1) = h(1);
H = sptoeplitz(col,lin);  % sparse matrix
%----------------------------------------------------------

%----------------------------------------------
%----- Generation of x (sparse)       ------------
%----------------------------------------------
xtrue = zeros(N,1);    % true spikes
step = N/3;
dist_min = floor(step/6);        % minimal distance between consecutive spikes
steps = randraw('po',dist_min,[N,1]); 
steps(find(steps==0))=[];        % distances between consecutive spikes
loc_spike=steps(1);
i=1;
indtrue = [];
while (loc_spike<N),
    indtrue = [indtrue, loc_spike];
    i=i+1;
    loc_spike = loc_spike+steps(i);
end;
NB_GAUSS = i-1;
disp(sprintf('There are %d Gaussian features = %.2g %% of the number of samples',...
             NB_GAUSS,NB_GAUSS/N*100));
xtrue(indtrue) = randn(NB_GAUSS,1);


%----------------------------------------------
%----- Generation of noisy data h*x+n 
%----------------------------------------------
SNR = 20;
y = H*xtrue;
z = adgnoise(y,SNR);

%----------------------------------------------
%----- Plot noisy data
%----------------------------------------------
figure(1), 
set(gca,'FontSize',16);
f1=plot(z,'k'); grid on; 
extraz = .1*(max(z)-min(z));
axis([0,length(z)-1,min(z)-extraz,max(z)+extraz]);
title(sprintf('%d Gaussian features - SNR = %d dB',NB_GAUSS,SNR));
disp('SIMULATED DATA'); 
disp('PRESS KEY TO CONTINUE'); 

%----------------------------------------------
%----- Run SBR 
%----------------------------------------------
disp('--SBR');
[x_std,Jit,FLAG1] = sbr_ld(H,z,mu,1,[],'all');

%----------------------------------------------
%----- Plot SBR results
%----------------------------------------------
figure(2);  % spikes
set(gca,'FontSize',16)
f1=plot(indtrue,xtrue(indtrue),'ok'); hold on; 
[x,y,f2]=spike(x_std); grid on; hold off
legend([f2,f1],'x*','x');
set(legend,'FontSize',18)
title('SBR recovery');

y_rec = H*full(x_std);  % SBR approximation

figure(1);  % spikes and approximation
hold on; 
f2=plot(y_rec,'-r','linewidth',2); grid on; hold on;
set(legend,'FontSize',20)
legend('Data','SBR approximation');

